from .select_funcs import (_get_selected_path_from_tree, _get_first_apply_checked_from_tree, _get_selected_path_from_list, _on_tree_selection_changed, _pick_preview_target)
from .files_funcs import (_clear_files_tree, _add_file_row, _fill_files_tree, _collect_checked_files, _open_file_from_row, get_files)
from .edit_funcs import (_iter_tree_rows, _gather_checked, get_files, append_log, get_all_files, get_hunks, get_all_subs, get_test_diff, get_nufiles, output_test, _ask_user_to_pick_file, _preview_for_path, preview_patch, _selected_tree_row_flags, _first_overwrite_checked, save_patch, save_all_checked, apply_diff_to_directory)



from ..imports import *
def _collect_table_data(self, table):
    data = {}
    for r in range(table.rowCount()):
        key_item = table.item(r, 0)
        if not key_item or not key_item.text().strip():
            continue
        val_item = table.item(r, 1)
        data[key_item.text().strip()] = val_item.text().strip() if val_item else ""
    return data

def _collect_headers(self):
    headers = {}
    for r in range(self.headers_table.rowCount()):
        # only include checked rows
        chk = self.headers_table.item(r, 0)
        if not chk or chk.checkState() != Qt.CheckState.Checked:
            continue
        key_item = self.headers_table.item(r, 1)
        key = key_item.text().strip() if key_item else ""
        value_cb = self.headers_table.cellWidget(r, 2)
        if value_cb:
            val = value_cb.currentText().strip()
        else:
            val_item = self.headers_table.item(r, 2)
            val = val_item.text().strip() if val_item else ""
        # if user picked/typed a MIME but left key blank, assume Content-Type
        if val and not key:
            key = "Content-Type"
            if key_item is None:
                key_item = QTableWidgetItem(key)
                self.headers_table.setItem(r, 1, key_item)
            else:
                key_item.setText(key)
        if key:
            headers[key] = val
    return headers



from ...imports import *
def on_tree_copy(self, paths: List[str]):
   """
   Called when the “Copy Selected” button is pressed.
   We log how many items, then forward to drop_area.
   """
   self._log(f"Copy Selected triggered on {len(paths)} path(s).")
   self.drop_area.process_files(paths)



##from .functionsTab import _FunctionsTab
from .apiTab import apiTab, startApiConsole
from .clipitTab import clipitTab, startClipitConsole
from .finderTab import finderConsole, startFinderConsole
from .logPaneTab import logPaneTab, startLogPaneConsole
from .appRunnerTab import appRunnerTab, startAppRunnerConsole
from .reactRunnerTab import reactRunnerTab, startReactRunnerConsole
from .windowManagerTab import windowManagerTab, startWindowManagerConsole
from .launcherWindowTab import launcherWindowTab, startLauncherWindowConsole
from abstract_gui.QT6 import QTabWidget,QMainWindow
from abstract_gui.QT6.utils.console_utils import ConsoleBase
from abstract_gui.QT6.utils.console_utils import startConsole
# Content Finder = the nested group you built (Find Content, Directory Map, Collect, Imports, Diff)
class ideTab(ConsoleBase):
    def __init__(self, *, bus=None, parent=None):
        super().__init__(bus=bus, parent=parent)
        inner = QTabWidget()
        self.layout().addWidget(inner)
        # all content tabs share THIS console’s bus
        inner.addTab(reactRunnerTab(),      "react Runner")
        inner.addTab(finderConsole(),   "Finder")
        inner.addTab(apiTab(),   "Api")
        inner.addTab(clipitTab(),   "Clipit")
        inner.addTab(windowManagerTab(),   "Window Mgr")
        inner.addTab(launcherWindowTab(),   "Launcher")




from abstract_gui.QT6 import *
from initFuncs import initFuncs
# New Tab: Directory Map
class finderTab(ConsoleBase):
    def __init__(self,  *, bus=None, parent=None):
        super().__init__(bus=bus, parent=parent)
        inner = QTabWidget()
        self.layout().addWidget(inner)
    

        grid = QGridLayout()
        install_common_inputs(
            self, grid, bus=bus,
            primary_btn=("Run search", self.start_search)
        )


        # Output area
        
        self.layout().addWidget(QLabel("Results"))
        self.list = QListWidget()
        self.list.itemDoubleClicked.connect(self.open_one)
        self.layout().addWidget(self.list, stretch=3)
        self._last_results = []


finderTab = initFuncs(finderTab)
startConsole(finderTab)




from ..imports import *

def start_map(self):
    self.btn_run.setEnabled(False)
    try:
        self.params = make_params(self)
    except Exception as e:
        QMessageBox.critical(self, "Bad input", str(e))
        self.btn_run.setEnabled(True)
        return

    class MapWorker(QThread):
        log = pyqtSignal(str)
        done = pyqtSignal(list)   # emit list of results instead of plain str

        def __init__(self, params):
            super().__init__()
            self.params = params

        def run(self):
            try:
                # assume get_directory_map returns str -> split into lines
                map_str = get_directory_map(**self.params)
                results = [line for line in map_str.splitlines() if line.strip()]
                self.done.emit(results)
            except Exception:
                tb = traceback.format_exc()
                self.log.emit(tb)
                self.done.emit([])

    self.worker = MapWorker(self.params)
    self.worker.log.connect(self.append_log)      # errors → logs
    self.worker.done.connect(self.display_map)    # results → results list
    self.worker.finished.connect(lambda: self.btn_run.setEnabled(True))
    self.worker.start()


def append_log(self, text: str):
    """Append error/debug text into logs pane."""
    self.log.moveCursor(self.log.textCursor().MoveOperation.End)
    self.log.insertPlainText(text)
    self.log.ensureCursorVisible()


def display_map(self, results: list[str]):
    """Display map results in the Results list widget."""
    self.list.clear()
    if results:
        self.list.addItems(results)
    else:
        self.append_log("No map generated.\n")



from abstract_gui.QT6 import *
import os
# — UI helpers —
def enable_widget(parent, name: str, enabled: bool):
    try:
        getattr(parent, name).setEnabled(enabled)
    except AttributeError:
        print(f"[WARN] No widget {name} in {parent}")

# — Actions —
def start_search(self):
 
    enable_widget(self,"btn_run",False)
    try:
        params = make_params(self)
    except Exception as e:
        QMessageBox.critical(self, "Bad input", str(e))

        enable_widget(self,"btn_run",True)
        return
    self.worker = SearchWorker(params)
    self.worker.log.connect(self.append_log)
    self.worker.done.connect(self.populate_results)
    self.worker.finished.connect(lambda: enable_widget(self,"btn_run",True))
    self.worker.start()
def append_log(self, text: str):
    self.log.moveCursor(self.log.textCursor().MoveOperation.End)
    self.log.insertPlainText(text)

def populate_results(self, results: list):
    self._last_results = results or []
    if not results:
        self.append_log("✅ No matches found.\n")
        enable_widget(self, "btn_secondary", False)
        return

    self.append_log(f"✅ Found {len(results)} file(s).\n")
    enable_widget(self, "btn_secondary", True)

    for fp in results:
        if isinstance(fp, dict):
            file_path = fp.get("file_path")
            lines = fp.get("lines", [])
        else:
            file_path = fp
            lines = []

        if not isinstance(file_path, str):
            continue

        if lines:
            for obj in lines:
                line = obj.get("line")
                text = f"{file_path}:{line}" if line is not None else file_path
                item = QListWidgetItem(text)
                item.setData(Qt.ItemDataRole.UserRole, {"file_path": file_path, "line": line})
                self.list.addItem(item)
                self.append_log(text + "\n")
        else:
            item = QListWidgetItem(file_path)
            item.setData(Qt.ItemDataRole.UserRole, {"file_path": file_path, "line": None})
            self.list.addItem(item)
            self.append_log(file_path + "\n")



# collapsible_log_panel.py
from __future__ import annotations
from abstract_gui.QT6 import *

class CollapsibleLogPanel(QtWidgets.QWidget):
    """
    Collapsible panel that *adds* height to the top-level window when expanded,
    and subtracts it when collapsed, so the central area isn't squished.
    """
    toggled = QtCore.pyqtSignal(bool)

    def __init__(self, title: str, content_widget: QtWidgets.QWidget,
                 *, start_visible: bool = True, parent: QtWidgets.QWidget | None = None):
        super().__init__(parent)
        self._content = content_widget
        self._frame = QtWidgets.QFrame(self)
        self._frame.setFrameShape(QtWidgets.QFrame.Shape.NoFrame)

        # make the frame fixed-height so layout doesn’t steal from central area
        self._frame.setSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred,
                                  QtWidgets.QSizePolicy.Policy.Fixed)

        fl = QtWidgets.QVBoxLayout(self._frame)
        fl.setContentsMargins(0, 0, 0, 0)
        fl.addWidget(self._content)

        # header as a toolbar
        self.toolbar = QtWidgets.QToolBar(self)
        self.toolbar.setIconSize(QtCore.QSize(16, 16))
        self.toolbar.setMovable(False)
        self.toolbar.setFloatable(False)
        self.toolbar.setToolButtonStyle(QtCore.Qt.ToolButtonStyle.ToolButtonTextBesideIcon)

        self.toggle_action = QtGui.QAction(self)
        self.toggle_action.setCheckable(True)
        self.toggle_action.setChecked(start_visible)
        self._title = title
        self._refresh_toggle_text(start_visible)
        self.toggle_action.triggered.connect(self._on_toggled)

        clear_action = QtGui.QAction("Clear", self)
        clear_action.triggered.connect(self._try_clear)

        self.toolbar.addAction(self.toggle_action)
        self.toolbar.addSeparator()
        self.toolbar.addAction(clear_action)
        self.toolbar.addSeparator()

        # keyboard shortcuts for toggle
        QtGui.QShortcut(QtGui.QKeySequence("Ctrl+L"), self, activated=self.toggle_action.trigger)
        QtGui.QShortcut(QtGui.QKeySequence("F12"), self, activated=self.toggle_action.trigger)

        # compose
        root = QtWidgets.QVBoxLayout(self)
        root.setContentsMargins(0, 0, 0, 0); root.setSpacing(0)
        root.addWidget(self.toolbar)
        root.addWidget(self._frame)

        # initial visibility + prepared height
        self._frame.setVisible(start_visible)
        QtCore.QTimer.singleShot(0, self._sync_frame_height)

    # public helpers
    def expand(self): self.toggle_action.setChecked(True)
    def collapse(self): self.toggle_action.setChecked(False)
    def is_expanded(self) -> bool: return self.toggle_action.isChecked()
    def content_widget(self) -> QtWidgets.QWidget: return self._content

    # internals
    def _refresh_toggle_text(self, expanded: bool):
        arrow = "▾" if expanded else "▸"
        self.toggle_action.setText(f"{self._title} {arrow}")

    def _current_frame_hint(self) -> int:
        # the height we want when expanded
        # you can cap this if you want: return min(self._frame.sizeHint().height(), 240)
        return self._frame.sizeHint().height()

    def _sync_frame_height(self):
        # keep the frame’s max/min height consistent with its visibility
        if self._frame.isVisible():
            h = self._current_frame_hint()
            self._frame.setMinimumHeight(h)
            self._frame.setMaximumHeight(h)
        else:
            self._frame.setMinimumHeight(0)
            self._frame.setMaximumHeight(0)

    def _resize_top_level_by(self, delta_h: int):
        if delta_h == 0:
            return
        win = self.window()
        if not isinstance(win, QtWidgets.QWidget):
            return
        g = win.geometry()
        new_h = max(win.minimumHeight(), g.height() + delta_h)

        # keep within screen’s available geometry
        screen = win.screen() or QtGui.QGuiApplication.primaryScreen()
        if screen:
            avail = screen.availableGeometry()
            # leave a small margin from the bottom
            max_h = avail.height() - 16
            new_h = min(new_h, max_h)

        win.resize(g.width(), new_h)

    @QtCore.pyqtSlot(bool)
    def _on_toggled(self, checked: bool):
        # compute delta before we change visibility
        if checked:
            target_h = self._current_frame_hint()
            prev_h = 0 if not self._frame.isVisible() else self._frame.height()
            delta = target_h - prev_h
        else:
            prev_h = self._frame.height() if self._frame.isVisible() else 0
            delta = -prev_h

        self._frame.setVisible(checked)
        self._refresh_toggle_text(checked)
        self._sync_frame_height()

        # do the window resize after layout settles
        QtCore.QTimer.singleShot(0, lambda d=delta: self._resize_top_level_by(d))
        self.toggled.emit(checked)

    def _try_clear(self):
        w = self._content
        # try common editors
        for cls in (QtWidgets.QPlainTextEdit, QtWidgets.QTextEdit):
            edit = w if isinstance(w, cls) else w.findChild(cls)
            if edit:
                edit.clear()
                return
